@@ -5,6 +5,8 @@ import re
 import requests
 import warnings
 import wget
+import tarfile
+import zipfile
 
 
 def validate_url(url):
@@ -24,6 +26,41 @@ def validate_url(url):
     return re.match(regex, url) is not None
 
 
+def make_tarfile(output_file, source_dir):
+    """tar a directory
+    args
+    -----
+    output_file: path to output file
+    source_dir: path to source directory
+
+    returns
+    -----
+    tarred directory will be in output_file
+    """
+    with tarfile.open(output_file, "w:gz") as tar:
+        tar.add(source_dir, arcname=os.path.basename(source_dir))
+
+
+def make_zipfile(path, ziph):
+    # ziph is zipfile handle
+    for root, dirs, files in os.walk(path):
+        for file in files:
+            ziph.write(os.path.join(root, file),
+                       os.path.relpath(os.path.join(root, file),
+                                       os.path.join(path, '..')))
+
+
+class BearerAuth(requests.auth.AuthBase):
+    """Bearer Authentication"""
+
+    def __init__(self, token):
+        self.token = token
+
+    def __call__(self, r):
+        r.headers["authorization"] = "Bearer " + self.token
+        return r
+
+
 class Client(object):
     """Zenodo Client object
 
@@ -42,12 +79,19 @@ class Client(object):
         ```
     """
 
-    def __init__(self, title=None, bucket=None, deposition_id=None, sandbox=None, ACCESS_TOKEN=None):
+    def __init__(self, title=None, bucket=None, deposition_id=None, sandbox=None, token=None):
+        """initialization method"""
+        if sandbox:
+            self._endpoint = "https://sandbox.zenodo.org/api"
+        else:
+            self._endpoint = "https://zenodo.org/api"
+
         self.title = title
         self.bucket = bucket
         self.deposition_id = deposition_id
         self.sandbox = sandbox
-        self.ACCESS_TOKEN = ACCESS_TOKEN
+        self._token = self._read_from_config if token is None else token
+        self._bearer_auth = BearerAuth(self._token)
         # 'metadata/prereservation_doi/doi'
 
     def __repr__(self):
@@ -109,60 +153,24 @@ class Client(object):
                         config[key] = value.strip()
         return config
 
-    def _get_baseurl(self):
-        """API URL
-
-        This either returns the sandbox API URL
-        or the actual Zenodo API URL.
-        Sandbox is used for testing purposes
-        and is not meant for projects you want to persist
-
+    @property
+    def _read_from_config(self):
+        """reads the web3.storage token from configuration file
+        configuration file is ~/.web3_storage_token
         Returns:
-            str: The API's URL
+            str: ACCESS_TOKEN to connect to web3 storage
         """
         if self.sandbox:
-            return "https://sandbox.zenodo.org/api"
+            dotrc = os.environ.get("ACCESS_TOKEN-sandbox", os.path.expanduser("~/.zenodo_token"))
         else:
-            return "https://zenodo.org/api"
+            dotrc = os.environ.get("ACCESS_TOKEN", os.path.expanduser("~/.zenodo_token"))
 
-    def _get_key(self):
-        """gets the ACCESS_TOKEN
-
-        Returns:
-            str: ACCESS_TOKEN to connect to zenodo
-        """
-        dotrc = os.environ.get("ACCESS_TOKEN", os.path.expanduser("~/.zenodo_token"))
         if os.path.exists(dotrc):
             config = self._read_config(dotrc)
-            if self.sandbox is None:
-                key = config.get("ACCESS_TOKEN")
-                return key
-            else:
-                key = config.get("ACCESS_TOKEN-sandbox")
-                return key
+            key = config.get("ACCESS_TOKEN-sandbox") if self.sandbox else config.get("ACCESS_TOKEN")
+            return key
         else:
-            print(' ** No ACCESS_TOKEN was found, check your ~/.zenodo_token file ** ')
-
-        # initialize with key
-        if self.ACCESS_TOKEN is not None:
-            print(' ** Using ACCESS_TOKEN supplied at initialization ** ')
-            return self.ACCESS_TOKEN
-
-    def _get_headers(self):
-        """gets the request headers
-
-        Returns:
-            dict: headers supplied to request calls
-        """
-        # gets the API key (tested)
-        key = self._get_key()
-
-        # header data attached to request
-        if key is not None:
-            return {"Content-Type": "application/json",
-                    "Authorization": f"Bearer {key}"}
-        else:
-            return None
+            print(' ** No token was found, check your ~/.zenodo_token file ** ')
 
     def _get_depositions(self):
         """gets the current project deposition
@@ -172,15 +180,9 @@ class Client(object):
         Returns:
             dict: dictionary containing project details
         """
-        # api baseurl (tested)
-        baseurl = self._get_baseurl()
-
-        # header data attached to request
-        headers = self._get_headers()
-
         # get request, returns our response
-        r = requests.get(f"{baseurl}/deposit/depositions",
-                         headers=headers)
+        r = requests.get(f"{self._endpoint}/deposit/depositions",
+                         auth=self._bearer_auth)
         if r.ok:
             return r.json()
         else:
@@ -197,16 +199,10 @@ class Client(object):
         Returns:
             dict: dictionary containing project details
         """
-        # api baseurl
-        baseurl = self._get_baseurl()
-
-        # header data attached to request
-        headers = self._get_headers()
-
         # get request, returns our response
         # if dep_id is not None:
-        r = requests.get(f"{baseurl}/deposit/depositions/{dep_id}",
-                         headers=headers)
+        r = requests.get(f"{self._endpoint}/deposit/depositions/{dep_id}",
+                         auth=self._bearer_auth)
 
         if r.ok:
             return r.json()
@@ -221,16 +217,9 @@ class Client(object):
         Returns:
             dict: dictionary containing project details
         """
-        # api baseurl
-        baseurl = self._get_baseurl()
-
-        # header data attached to request
-        headers = self._get_headers()
-        dep_id = self.deposition_id
-
         # get request, returns our response
-        r = requests.get(f"{baseurl}/deposit/depositions/{dep_id}/files",
-                         headers=headers)
+        r = requests.get(f"{self._endpoint}/deposit/depositions/{self.deposition_id}/files",
+                         auth=self._bearer_auth)
 
         if r.ok:
             return r.json()
@@ -248,19 +237,12 @@ class Client(object):
         Returns:
             str: the bucket URL to upload files to
         """
-        # api baseurl
-        baseurl = self._get_baseurl()
-
-        # header data attached to request
-        headers = self._get_headers()
-
         dic = self.list_projects
-
         dep_id = dic[title] if dic is not None else None
 
         # get request, returns our response, this the records metadata
-        r = requests.get(f"{baseurl}/deposit/depositions/{dep_id}",
-                         headers=headers)
+        r = requests.get(f"{self._endpoint}/deposit/depositions/{dep_id}",
+                         auth=self._bearer_auth)
 
         if r.ok:
             return r.json()['links']['bucket']
@@ -278,18 +260,9 @@ class Client(object):
         Returns:
             str: the bucket URL to upload files to
         """
-        # api baseurl
-        baseurl = self._get_baseurl()
-
-        # header data attached to request
-        headers = self._get_headers()
-
-        # dic = get_projects()
-        # dep_id = dic[title]
-
         # get request, returns our response
-        r = requests.get(f"{baseurl}/deposit/depositions/{dep_id}",
-                         headers=headers)
+        r = requests.get(f"{self._endpoint}/deposit/depositions/{dep_id}",
+                         auth=self._bearer_auth)
 
         if r.ok:
             return r.json()['links']['bucket']
@@ -297,14 +270,8 @@ class Client(object):
             return None
 
     def _get_api(self):
-        # api baseurl
-        baseurl = self._get_baseurl()
-
-        # header data attached to request
-        headers = self._get_headers()
-
         # get request, returns our response
-        r = requests.get(f"{baseurl}", headers=headers)
+        r = requests.get(f"{self._endpoint}", auth=self._bearer_auth)
 
         if r.ok:
             return r.json()
@@ -405,15 +372,9 @@ class Client(object):
                           UserWarning)
             upload_type = 'other'
 
-        # api baseurl
-        baseurl = self._get_baseurl()
-
-        # header data attached to request
-        headers = self._get_headers()
-
         # get request, returns our response
-        r = requests.post(f"{baseurl}/deposit/depositions",
-                          headers=headers,
+        r = requests.post(f"{self._endpoint}/deposit/depositions",
+                          auth=self._bearer_auth,
                           data=json.dumps({}))
 
         if r.ok:
@@ -468,11 +429,6 @@ class Client(object):
         Returns:
             dict: dictionary with new metadata
         """
-
-        baseurl = self._get_baseurl()
-
-        headers = self._get_headers()
-
         if upload_type is None:
             upload_type = 'other'
 
@@ -487,8 +443,8 @@ class Client(object):
             }
         }
 
-        r = requests.put(f"{baseurl}/deposit/depositions/{dep_id}",
-                         headers=headers,
+        r = requests.put(f"{self._endpoint}/deposit/depositions/{dep_id}",
+                         auth=self._bearer_auth,
                          data=json.dumps(data))
 
         if r.ok:
@@ -508,62 +464,217 @@ class Client(object):
         if not Path(os.path.expanduser(file_path)).exists():
             print(f"{file_path} does not exist. Please check you entered the correct path")
         else:
-            key = self._get_key()
             bucket_link = self.bucket
 
-            # parameters
-            params = {'access_token': key}
-
             with open(file_path, "rb") as fp:
                 # text after last '/' is the filename
                 filename = file_path.split('/')[-1]
                 r = requests.put(f"{bucket_link}/{filename}",
-                                 params=params,
+                                 auth=self._bearer_auth,
                                  data=fp,)
 
                 print(f"{file_path} successfully uploaded!") if r.ok else print("Oh no! something went wrong")
 
-    def download_file(self, filename=None):
+    def upload_zip(self, source_dir=None, output_file=None):
+        """upload a directory to a project as zip
+
+        This will: 
+            1. zip the directory, 
+            2. upload the zip directory to your project
+            3. remove the zip file from your local machine
+
+        Args:
+            source_dir (str): path to directory to tar
+            output_file (str): name of output file (optional)
+                defaults to using the source_dir name as output_file
+        """
+        # make sure source directory exists
+        source_dir = os.path.expanduser(source_dir)
+        source_obj = Path(source_dir)
+        if not source_obj.exists():
+            raise FileNotFoundError(f"{source_dir} does not exist")
+
+        # acceptable extensions for outputfile
+        acceptable_extensions = ['.zip']
+
+        # use name of source_dir for output_file if none is included
+        if not output_file:
+            output_file = f"{source_obj.stem}.zip"
+            output_obj = Path(output_file)
+        else:
+            output_file = os.path.expanduser(output_file)
+            output_obj = Path(output_file)
+            extension = ''.join(output_obj.suffixes)  # gets extension like .tar.gz
+            # make sure extension is acceptable
+            if extension in acceptable_extensions:
+                raise Exception(f"Extension must be in {acceptable_extensions}")
+            # add an extension if not included
+            if not extension:
+                output_file = os.path.expanduser(output_file + '.zip')
+                output_obj = Path(output_file)
+
+        # check to make sure outputfile doesn't already exist
+        if output_obj.exists():
+            raise Exception(f"{output_obj} already exists. Please chance the name")
+
+        # create tar directory if does not exist
+        if output_obj.parent.exists():
+            with zipfile.ZipFile(output_file, 'w', zipfile.ZIP_DEFLATED) as zipf:
+                make_zipfile(source_dir, zipf)
+        else:
+            os.makedirs(output_obj.parent)
+            with zipfile.ZipFile(output_file, 'w', zipfile.ZIP_DEFLATED) as zipf:
+                make_zipfile(source_dir, zipf)
+
+        # upload the file
+        self.upload_file(file_path=output_file)
+
+        # remove tar file after uploading it
+        os.remove(output_file)
+
+    def upload_tar(self, source_dir=None, output_file=None):
+        """upload a directory to a project
+
+        This will: 
+            1. tar the directory, 
+            2. upload the tarred directory to your project
+            3. remove the tar file from your local machine
+
+        Args:
+            source_dir (str): path to directory to tar
+            output_file (str): name of output file (optional)
+                defaults to using the source_dir name as output_file
+        """
+        # output_file = './tmp/tarTest.tar.gz'
+        # source_dir = '/Users/gloege/test'
+
+        # make sure source directory exists
+        source_dir = os.path.expanduser(source_dir)
+        source_obj = Path(source_dir)
+        if not source_obj.exists():
+            raise FileNotFoundError(f"{source_dir} does not exist")
+
+        # acceptable extensions for outputfile
+        acceptable_extensions = ['.tar.gz']
+
+        # use name of source_dir for output_file if none is included
+        if not output_file:
+            output_file = f"{source_obj.stem}.tar.gz"
+            output_obj = Path(output_file)
+        else:
+            output_file = os.path.expanduser(output_file)
+            output_obj = Path(output_file)
+            extension = ''.join(output_obj.suffixes)  # gets extension like .tar.gz
+            # make sure extension is acceptable
+            if extension in acceptable_extensions:
+                raise Exception(f"Extension must be in {acceptable_extensions}")
+            # add an extension if not included
+            if not extension:
+                output_file = os.path.expanduser(output_file + '.tar.gz')
+                output_obj = Path(output_file)
+
+        # check to make sure outputfile doesn't already exist
+        if output_obj.exists():
+            raise Exception(f"{output_obj} already exists. Please chance the name")
+
+        # create tar directory if does not exist
+        if output_obj.parent.exists():
+            make_tarfile(output_file=output_file, source_dir=source_dir)
+        else:
+            os.makedirs(output_obj.parent)
+            make_tarfile(output_file=output_file, source_dir=source_dir)
+
+        # upload the file
+        self.upload_file(file_path=output_file)
+
+        # remove tar file after uploading it
+        os.remove(output_file)
+
+    def download_file(self, filename=None, dst_path=None):
         """download a file from project
 
         Args:
             filename (str): name of the file to download
+            dst_path (str): destination path to download the data (default is current directory)
         """
         if filename is None:
             print(" ** filename not supplied ** ")
 
-        key = self._get_key()
-
-        # parameters
-        params = {'access_token': key}
-
         bucket_link = self.bucket
 
         if bucket_link is not None:
             if validate_url(bucket_link):
                 r = requests.get(f"{bucket_link}/{filename}",
-                                 params=params,
-                                 )
-                wget.download(r.url) if r.ok else print(f" ** Something went wrong, check that {filename} is in your poject  ** ")
+                                 auth=self._bearer_auth)
+
+                # if dst_path is not set, set download to current directory
+                # else download to set dst_path
+                if dst_path is None:
+                    wget.download(r.url) if r.ok else print(f" ** Something went wrong, check that {filename} is in your poject  ** ")
+                elif os.path.isdir(dst_path):
+                    cwd = os.getcwd()
+                    os.chdir(dst_path)
+                    wget.download(r.url) if r.ok else print(f" ** Something went wrong, check that {filename} is in your poject  ** ")
+                    os.chdir(cwd)
+                else:
+                    raise FileNotFoundError(f'{dst_path} does not exist')
             else:
                 print(f' ** {bucket_link}/{filename} is not a valid URL ** ')
 
+    def _is_doi(self, string=None):
+        """test if string is of the form of a zenodo doi
+        10.5281.zenodo.[0-9]+
+
+        Args:
+            string (strl): string to test. Defaults to None.
+
+        Returns:
+           bool: true is string is doi-like
+        """
+        import re
+        pattern = re.compile("10.5281/zenodo.[0-9]+")
+        return pattern.match(string)
+
+    def _get_record_id_from_doi(self, doi=None):
+        """return the record id for given doi
+
+        Args:
+            doi (string, optional): the zenodo doi. Defaults to None.
+
+        Returns:
+            str: the record id from the doi (just the last numbers)
+        """
+        return doi.split('.')[-1]
+
+    def get_urls_from_doi(self, doi=None):
+        """the files urls for the given doi
+
+        Args:
+            doi (str): the doi you want the urls from. Defaults to None.
+
+        Returns:
+            list: a list of the files urls for the given doi
+        """
+        if self._is_doi(doi):
+            record_id = self._get_record_id_from_doi(doi)
+        else:
+            print(f"{doi} must be of the form: 10.5281/zenodo.[0-9]+")
+
+        # get request (do not need to provide access token since public
+        r = requests.get(f"https://zenodo.org/api/records/{record_id}")  # params={'access_token': ACCESS_TOKEN})
+        return [f['links']['self'] for f in r.json()['files']]
+
     def delete_file(self, filename=None):
         """delete a file from a project
 
         Args:
             filename (str): the name of file to delete
         """
-        key = self._get_key()
-
-        # parameters
-        params = {'access_token': key}
-
         bucket_link = self.bucket
 
         # with open(file_path, "rb") as fp:
         _ = requests.delete(f"{bucket_link}/{filename}",
-                            params=params,)
+                            auth=self._bearer_auth)
 
     def _delete_project(self, dep_id=None):
         """delete a project from repository by ID
@@ -571,18 +682,11 @@ class Client(object):
         Args:
             dep_id (str): The project deposition ID
         """
-        # warnings.warn("This will permanently delete your project. Proceed with caution'", )
-        # api baseurl
-        baseurl = self._get_baseurl()
-
-        # header data attached to request
-        headers = self._get_headers()
-
         print('')
         # if input("are you sure you want to delete this project? (y/n)") == "y":
         # delete requests, we are deleting the resource at the specified URL
-        r = requests.delete(f'{baseurl}/deposit/depositions/{dep_id}',
-                            headers=headers)
+        r = requests.delete(f'{self._endpoint}/deposit/depositions/{dep_id}',
+                            auth=self._bearer_auth)
         # response status
         print(r.status_code)
 
